# $Date: 1996/07/02 14:16:33 $ $Author: stefanw $ $Revision: 1.4 $ #
# #
# stefanw #

#++
interp.mu

	numeric::lagrange - lagrange interpolation of data 
			
	lagrange(pointlist,valuelist,x <,F>)

	pointlist - list of evaluation points
	valuelist - list of the values at the evaluation points
	x	  - variable 
	F	  - (optional) coefficient ring

Returns the polynomial of lowest degree which interpolates the given data.

Note that if F is not a field, the interpolation may be impossible.
F::convert must be able to handle the operands of pointlist and valuelist.
++#

numeric::lagrange:=proc()
local i,n,t,k,a,pointlist,valuelist,x,F,result;

begin
if testargs() then
	if args(0)<3 or args(0)>5 then
		 error("wrong number of arguments");
	elif {domtype(args(1)),domtype(args(2))}<>{DOM_LIST} then
		error("First and second argument must be lists");
	elif nops(args(1))<>nops(args(2)) then
		error("Number of points does not equal number of values");
	elif nops(args(1)) <> nops({op(args(1))}) then
		error("Evaluation points should be distinct");
	elif domtype(args(3))<> DOM_IDENT then
		error("Third argument must be variable");
	elif args(0)=4 then
		if args(4)=hold(Expr) then
			# ok, do nothing #	
		elif (args(4))::hasProp(Cat::Ring) <> TRUE then
			error("Fourth argument must be a Cat::Ring");
	        end_if
	end_if
end_if;			
pointlist:=args(1);
valuelist:=args(2);
x:=args(3);
if args(0)=3 then 
	F:=hold(Expr);
else
	F:=args(4);
end_if;
n:=nops(pointlist)-1;

a:=array(0..n);
t:=array(0..n);
for i from 0 to n do
	t[i]:=valuelist[i+1];
	k:=i-1;
	while k>=0 do
		t[k]:=(t[k+1]-t[k])/(pointlist[i+1]-pointlist[k+1]);
		k:=k-1;
	end_while;
	a[i]:=t[0];
end_for;


result:=poly(a[n],[x],F);
k:=n-1;
while k>=0 do
	result:=result*poly([[1,1],[-pointlist[k+1],0]],[x],F)+poly(a[k],[x],F);
	k:=k-1;
end_while;
result
end_proc:

# end of file #
