/**************************************************************************
**                         MPR_protocol.h                                **
** In diesem File werden die Typen f"ur das Protokoll MPR... definiert.  **
** Au\3erdem werden die Schnittstellenfunktionen deklariert.             **
**									 **
** Werden nur die Typ-Konstanten fuer die Meldungen benoetigt, so ist    **
** das Makro MPR_TYPE_CONST zu definieren.				 **
**************************************************************************/

#ifndef __MPR_protocol__
#define __MPR_protocol__

/**************************************************************************
** Typ-Konstanten fuer Meldungen                                         **
**************************************************************************/
#define MPRCno_of_messages	  27

#define MPRCgetstatus              0
#define MPRCinput                  1
#define MPRCoutput                 2
#define MPRCsystem                 3
#define MPRCgettextinput           4
#define MPRCgetinput               5
#define MPRCtextwidth              6
#define MPRCprettyprint            7
#define MPRChelp                   8
#define MPRCerror                  9
#define MPRCexample               10
#define MPRCmdxcommand            11
#define MPRCdisplayvariables      12
#define MPRCprompt		  13
#define MPRClongprompt      	  14
#define MPRCdebuggermode    	  15
#define MPRCmdxprompt             16
#define MPRC_mv_graphic_cmd       17
#define MPRC_vx_plot_cmd          18
#define MPRC_vm_plot_cmd          19
#define MPRC_mv_graphic_error     20
#define MPRC_vx_graphic_error     21
#define MPRCsignal                22      /* Nachricht ist Signal (s.u.) */
#define MPRCmdxoutput		  23
#define MPRCmdxerror		  24
#define MPRCsyntaxerror		  25
#define MPRCincompleteinput	  26


/**************************************************************************
** MPRTtype                                                              **
** Instanzen dieses Typs geben an, um was f"ur einen Typ von Nachricht   **
** es sich bei dieser Nachricht handelt.                                 **
**************************************************************************/
typedef char MPRTtype ;


#ifndef MPR_TYPE_CONST

#define MEXDSTANDARD
#ifdef DOS_FILENAMES
#  include "MDE_decl.h"
#  include "MEX_ext.h"
#else
#  include "MDE_declare.h"
#  include "MEX_extern.h"
#endif

typedef int MPRTbool;
typedef	int MPRTstatus;

/**************************************************************************
** MPRTchannel                                                           **
** Auf Instanzen diesen Typs setzen die Funktionen des Protokolls auf.   **
** Eine Instanz besteht aus einem Zeiger auf ein FILE, aus dem die Daten **
** gelesen bzw. auf den sie geschrieben werden, einem Puffer zum         **
** zwischenpuffern von gelesenen Daten. Dieser wird ben"otigt, da einige **
** Bytes nur im Kontext mit den darauffolgenden Bytes interpretiert      **
** werden k"onnen. Au\3erdem beinhaltet die Instanz einen Z"ahler der    **
** angibt, wieviele Bytes im Puffer gespeichert sind.                    **
**************************************************************************/

#ifdef MSDOS
typedef struct _MPRSchannel {
    int dummy;
} MPRSchannel;
#else
typedef struct _MPRSchannel
{
	FILE	*file ;
	char	readbuffer[1] ;
	short	rsize ;
	int	bytes;
} MPRSchannel ;
#endif
typedef MPRSchannel *MPRTchannel ;


/**************************************************************************
** MPRTchar                                                              **
** Dies ist der Typ, der aus einer Instanz des Typs MPRTchannel          **
** herausgelesen wird. Eine Instanz dieses Typs enth"alt mehr            **
** Informationen als nur den Wert des Bytes, den er repr"asentiert.      **
** Sie enth"alt z.B. die Information, ob es sich "uberhaupt um ein Byte  **
** handelt oder um das Ende einer Nachricht. Sie kann auch weitere       **
** Informationen zum Ende der Nachricht enthalten. Es w"are auch         **
** m"oglich, in eine Nachricht Informationen einzuschieben, dies m"u\3te **
** das entsprechende Protokoll dann aber verstehen.                      **
**************************************************************************/
typedef short MPRTchar ;


/**************************************************************************
** Sonstige Konstanten                                                   **
**************************************************************************/
#define MPRCfail		30	  /* ioctl() fail auf Deskribtor */
#define MPRCnone		31	  /* keine bytes auf Deskribtor  */
#define MPRCread		32	  /* bytes auf Deskribtor        */


/*** Moegliche Signale des Typs MPRCsignal *******************************/
#define MPRC_SIG_break            'b' 
#define MPRC_SIG_quit             'q'
#define MPRC_SIG_kill             'k'


/*
 * Konstanten f"ur die Kommunikation mit VCAM
 *
 * Dazu einige Erl"auterungen:
 *
 * MPRC_mv_graphic_cmd      Kennzeichnet eine Botschaft von MuPAD an VCam,
 *                          in der ein Grafik-Befehl (= Grafikdaten + Befehl)
 *                          geschickt wird.
 *
 * MPRC_vx_plot_cmd         Kennzeichnet eine Botschaft von VCam an XMuPAD,
 *                          in der ein Plot-Befehl an XMuPAD geschickt wird,
 *                          der ins XMuPAD-Fenster geschrieben werden soll.
 *
 * MPRC_vm_plot_cmd         Kennzeichnet eine Botschaft von VCam an MuPAD,
 *                          in der dem Kern MuPAD ein Plot-Befehl geschickt
 *                          wird. Zur Zeit wird dieser Befehl wohl noch "uber
 *                          XMuPAD geschickt.
 *
 * MPRC_vx_graphic_error    Kennzeichnet eine Botschaft von VCam an XMuPAD, 
 *                          in der XMuPAD mitgeteilt wird, dass das Grafiktool
 *                          Probleme hat (bspw. konnte eine Datei nicht ge"off-
 *                          net werden).
 *
 * MPRC_mv_graphic_error    Kennzeichnet eine Botschaft vom Kern MuPAD an das
 *                          Grafiktool VCam, in der diesem mitgeteilt wird, das
 *                          es bei der Bearbeitung eines Plot-Befehles, welcher
 *                          von VCam an den Kern geschickt wurde, ein Problem
 *                          gab. Bisher kam dann immer die Fehlermeldung: 'An
 *                          error was detected in the definition of the objects.'
 */



/**************************************************************************
** Deklaration der Funktionalen Schnittstelle des Protokolls MPR...      **
**************************************************************************/
#ifndef MSDOS

MPRTchannel	MPR_createFP	(FILE *fp) ;
void		MPR_delete	(MPRTchannel ch) ;
MPRTtype	MPR_getstart	(MPRTchannel ch) ;
MPRTchar	MPR_get		(MPRTchannel ch) ;
MPRTchar	MPR_try		(MPRTchannel ch) ;
MPRTbool	MPR_EOM		(MPRTchar c) ;
MPRTbool	MPR_nochar	(MPRTchar c) ;
char		MPR_convert	(MPRTchar c) ;
void		MPR_flush	(MPRTchannel ch) ;
MPRTstatus	MPR_checkChannel(MPRTchannel ch) ;
void		MPR_putstart	(MPRTchannel ch, MPRTtype type) ;
void		MPR_putend	(MPRTchannel ch) ;
void		MPR_putchar	(MPRTchannel ch, char c) ;
void		MPR_putstring	(MPRTchannel ch, char *s) ;
void		MPR_putmessage	(MPRTchannel ch, MPRTtype type, char *s) ;
void		MPR_putsignal	(MPRTchannel ch, char signal) ;

#else
/* Unter Windows wird nur die Ausgabe-Routine MPR_putmessage verwendet.
 * Eingaben werden vom Windows-Rahmen verarbeitet. Alle Ausgabe gehen auf
 * einen Kanal, nur der Typ ist entscheidend. */

#define MPR_putmessage(ch,type,str) { MSW_pr_puts((type), (str)); }

#endif /* ! MSDOS */

#endif /* ! MPR_TYPE_CONST */
#endif /* __MPR_protocol__ */
